import json
from typing import (
    Any,
    Dict,
    Optional,
)

from messagebus import (
    AckMessage,
    Event,
    InternalErrorMessage,
    Message,
    MessageBus,
)
from messageprocessorutil import UuidMessageProcessor

from store_config.interactor import GetStoreConfigInteractor


class UpdateStoreConfigProcessor(UuidMessageProcessor):

    def __init__(
        self,
        interactor: GetStoreConfigInteractor,
    ) -> None:

        super().__init__()
        self.interactor = interactor

    def parse_data(
        self,
        data: Optional[bytes],
    ) -> None:

        return None

    def call_business(
        self,
        _,
    ) -> bool:

        return self.interactor.execute()

    def format_response(
        self,
        message_bus: MessageBus,
        message: Message,
        event: Event,
        request: Any,
        result: Dict,
    ) -> None:

        return message_bus.reply_message(
            message=message,
            reply_message=AckMessage(
                data=json.dumps(result).encode(),
            )
        )

    def format_exception(
        self,
        message_bus: MessageBus,
        message: Message,
        event: Event,
        input_obj: str,
        exception: Exception,
    ) -> None:

        message_bus.reply_message(
            message=message,
            reply_message=InternalErrorMessage(ex=exception)
        )

    def format_parse_exception(
        self,
        message_bus: MessageBus,
        message: Message,
        event: Event,
        data: bytes,
        exception: Exception,
    ) -> None:

        self.format_exception(
            message_bus=message_bus,
            message=message,
            event=event,
            input_obj=data.decode("utf-8"),
            exception=exception,
        )




from store_config.service import StoreConfigurationService


class UpdateStoreConfigInteractor:

    def __init__(
        self,
        store_config_service: StoreConfigurationService,
    ) -> None:

        self.store_config_service = store_config_service

    def execute(
        self,
    ) -> bool:

        return self.store_config_service.update_configuration()



import json
import logging
import logging.config
import os

import cfgtools
from helper import import_pydevd
from mbcontextmessagehandler import (
    MbContextMessageBus,
    MbContextMessageHandler,
)
from msgbus import MBEasyContext
from store_config.message_handler_builder import StoreConfigMessageHandlerBuilder
from store_config.message_handler_builder.config import Config


def default_log_config():
    return {
        "version": 1,
        "formatters": {
            "defaultFormatter": {
                "()": "helper.IsoFormatter"
            }
        },
        "handlers": {
            "stdout": {
                "class": "logging.StreamHandler",
                "formatter": "defaultFormatter",
                "level": "DEBUG",
                "stream": "ext://sys.stdout",
            },
            "stderr": {
                "class": "logging.StreamHandler",
                "formatter": "defaultFormatter",
                "level": "ERROR",
                "stream": "ext://sys.stderr",
            },
        },
        "loggers": {
            "msgbus": {
                "level": "INFO",
                "propagate": True,
            }
        },
        "root": {
            "level": "DEBUG",
            "handlers": [
                "stdout",
                "stderr"
            ]
        },
        "disable_existing_loggers": False
    }


logger = logging.getLogger(__name__)


def replace_env_vars(value: str):
    start, end = (value.find("{"), value.find("}"))
    if start != -1 and end != -1 and end > start + 1:
        var_name = value[start + 1:end]
        var_value = os.environ.get(var_name)
        if var_value is not None:
            return value[:start] + var_value + value[end + 1:]

    return value


def main():

    loader_path = os.environ["LOADERCFG"]
    if os.path.exists(loader_path):
        configuration = cfgtools.read(loader_path)
        import_pydevd(loader_path=loader_path, port=9185)
    else:
        configuration = cfgtools.read_string(loader_path)

    config = Config(store_config=configuration)

    log_path = os.environ["LOG_PATH"]
    try:
        log_dict = json.loads(log_path)
        if len(log_dict) == 0:
            log_dict = default_log_config()
    except ValueError:
        with open(log_path, "rb") as f:
            file_content = f.read().decode()
            log_dict = json.loads(file_content)

    for handler in log_dict.get("handlers", {}).values():
        file_name = handler.get("filename")
        if file_name:
            handler["filename"] = os.path.normpath(replace_env_vars(file_name))

    logging.config.dictConfig(log_dict)

    mb_context = MBEasyContext(name="StoreConfig")
    message_bus = MbContextMessageBus(mb_context=mb_context)

    message_handler_builder = StoreConfigMessageHandlerBuilder(config=config)
    message_handler = MbContextMessageHandler(
        message_bus=message_bus,
        service_name="StoreWideConfig",
        service_type="StoreWideConfig",
        required_services=[],
        message_handler_builder=message_handler_builder,
    )

    logger.info("Starting StoreConfig")
    return message_handler.handle_messages()


if __name__ == "__main__":
    main()



from cfgtools import Group
from messagehandler import MessageHandlerBuilder
from messageprocessor import (
    DefaultMessageProcessorExecutorFactory,
    MessageProcessorMessageHandler,
)
from messageprocessorutil import LoggingProcessorCallback
from store_config.processor import (
    GetConfigProcessor,
    GetFullConfigProcessor,
    SetConfigProcessor,
    GetStoreConfigProcessor,
    UpdateStoreConfigProcessor,
)
from store_config_api.shared.token import (
    TK_STORE_CFG_GET,
    TK_STORE_CFG_GET_FULL,
    TK_STORE_CFG_SET,
    TK_STORE_CFG_GET_COMPLETE_CONFIG,
    TK_STORE_CFG_UPDATE_COMPLETE_CONFIG,
)
from .config import Config
from ..interactor import (
    GetStoreConfigInteractor,
    UpdateStoreConfigInteractor,
)
from ..repository import StoreConfigurationRepository
from ..service import StoreConfigurationService


class StoreConfigMessageHandlerBuilder(MessageHandlerBuilder):

    def __init__(self, config: Config):

        self.config = config
        self.config_dict = {}
        for key, value in read_config_recursively(self.config.store_config.find_group("PublishedConfiguration")):
            self.config_dict[key] = value

        self.get_store_config_processor = None
        self.update_store_config_processor = None

    def build_singletons(self):
        store_configuration_repository = StoreConfigurationRepository(
            configs=self.config_dict,
        )
        store_config_service = StoreConfigurationService(
            store_config_repository=store_configuration_repository,
        )

        get_store_config_interactor = GetStoreConfigInteractor(
            store_config_service=store_config_service,
        )
        self.get_store_config_processor = GetStoreConfigProcessor(
            interactor=get_store_config_interactor,
        )
        update_store_config_interactor = UpdateStoreConfigInteractor(
            store_config_service=store_config_service,
        )
        self.update_store_config_processor = UpdateStoreConfigProcessor(
            interactor=update_store_config_interactor,
        )

    def build_message_handler(self, message, event):
        return MessageProcessorMessageHandler(
            event_processors=None,
            message_processors={
                TK_STORE_CFG_GET_COMPLETE_CONFIG: self.get_store_config_processor,
                TK_STORE_CFG_UPDATE_COMPLETE_CONFIG: self.update_store_config_processor,
                TK_STORE_CFG_GET: GetConfigProcessor(config_dict=self.config_dict),
                TK_STORE_CFG_SET: SetConfigProcessor(config_dict=self.config_dict),
                TK_STORE_CFG_GET_FULL: GetFullConfigProcessor(store_config=self.config.store_config),
            },
            executor_factory=DefaultMessageProcessorExecutorFactory(
                [
                    LoggingProcessorCallback(),
                ]
            ),
        )

    def destroy_message_handler(self, message_handler):
        return

    def destroy_singletons(self):
        return


def read_config_recursively(group: Group, parent_path: str = ""):
    if len(group.keys) > 0:
        for key in group.keys:
            value = key.values if len(key.values) != 1 else key.values[0]
            full_key = f"{parent_path}.{key.name}" if parent_path else key.name
            yield full_key, value

    for child_group in group.groups:
        path = child_group.name
        if parent_path != "":
            path = f"{parent_path}.{child_group.name}"

        for key, value in read_config_recursively(group=child_group, parent_path=path):
            yield key, value



from typing import (
    Dict,
    List,
    Optional,
    Union,
)

from messagebus import (
    AckMessage,
    InternalErrorMessage,
    NakMessage,
)
from messageprocessorutil import UuidMessageProcessor


class GetConfigProcessor(UuidMessageProcessor):
    def __init__(self, config_dict: Dict[str, Union[str, List[str]]]) -> None:
        super().__init__()
        self.store_config = config_dict

    def parse_data(self, data: bytes) -> str:
        return data.decode()

    def call_business(self, key_path: str) -> Optional[Union[str, List[str]]]:
        return self.store_config.get(key_path)

    def format_response(
        self,
        message_bus,
        message,
        event,
        key_path,
        value,
    ):
        if value is None:
            reply = NakMessage()
        else:
            ret = value
            if isinstance(value, list):
                ret = "\0".join(value)

            reply = AckMessage(data=ret.encode())

        message_bus.reply_message(
            message=message,
            reply_message=reply,
        )

    def format_exception(self, message_bus, message, event, input_obj, exception):
        message_bus.reply_message(
            message=message,
            reply_message=InternalErrorMessage(ex=exception),
        )

    def format_parse_exception(self, message_bus, message, event, data, exception):
        message_bus.reply_message(
            message=message,
            reply_message=InternalErrorMessage(ex=exception),
        )



import json
import logging
import zipfile

import requests

from pathlib import Path
from typing import (
    Any,
    Dict,
    Optional,
)

from store_config.utils import (
    jsons_dir,
    singleton,
    store_configs_path,
    storecfg_bundles_path,
)
from datetime import (
    datetime,
    date,
)

logger = logging.getLogger(__name__)


@singleton
class StoreConfigurationRepository:
    def __init__(
        self,
        configs,
    ) -> None:

        self.configs = configs

    def get_current_configuration(
        self,
    ) -> Dict:

        try:
            store_config = json.loads(self._read_current_configuration())
            pos_configs = self._get_pos_configs(store_config=store_config)
            jsons_content = self._get_jsons_content(jsons_dir)
            merged_jsons = {
                **store_config,
                **self.configs,
                **pos_configs,
                **jsons_content,
            }

            return merged_jsons

        except requests.exceptions.RequestException:
            logger.exception("Error while updating store configuration file")

            current_config = self._read_current_configuration()
            return json.loads(current_config)

    def update_store_configuration(self) -> bool:
        try:
            download_url = self._consult_external_api()
            zip_response = requests.get(download_url)
            zip_response.raise_for_status()
            self._update_config_file_from_zip(zip_response.content)
            return True

        except (Exception,):
            logger.exception("Failed do update config file")

        return False

    def _consult_external_api(
        self,
    ) -> str:

        store_id = self.configs.get("Store.Id")
        host = self.configs.get("BackOffice.ServerURL")
        url = f"{host}/pump/storeBundle/{store_id}"
        headers = {
            "x-api-key": self.configs.get("BackOffice.ApiKey"),
        }
        response = requests.get(url, headers=headers, allow_redirects=True)
        response.raise_for_status()
        download_url = response.text.strip().strip('"')
        return download_url

    def _update_config_file_from_zip(
        self,
        zip_content: str,
    ) -> Optional[dict]:

        try:
            self._extract_zip_config(zip_content)
            content = self._read_store_config_file()
            config = json.loads(content)
            config["lastStoreConfigUpdate"] = datetime.now().isoformat()
            self._write_store_config_file(config)
            return config

        except (Exception,):
            logger.exception("Error while saving store configuration file")

    @staticmethod
    def _extract_zip_config(
        zip_content: str,
    ) -> None:

        storecfg_bundles_path.mkdir(parents=True, exist_ok=True)
        zip_path = storecfg_bundles_path / "storeBundle.zip"
        with open(zip_path, "wb") as f:
            f.write(zip_content)

        with zipfile.ZipFile(zip_path, "r") as zip_ref:
            zip_ref.extractall(storecfg_bundles_path)

        zip_path.unlink(missing_ok=True)

    @staticmethod
    def _get_jsons_content(
        dir_path: Path,
    ) -> Dict[str, Any]:

        if not dir_path.is_dir():
            logger.info("No jsons directory found")
            return {}

        content = {}
        for file in dir_path.glob("*.json"):
            try:
                with file.open(encoding="utf-8") as f:
                    file_name = file.name.split(".json")[0]
                    file_content = json.load(f)
                    if file_name == "e2e":
                        content.update(file_content)
                    else:
                        content[file_name] = file_content

            except (Exception,):
                logger.exception(f"Failed to read {file.name}")

        return content

    def _read_current_configuration(self) -> str:

        try:
            if not store_configs_path.exists():
                self.update_store_configuration()

            content = self._read_store_config_file()
            if "pos_operator_level" not in json.loads(content):
                self.update_store_configuration()
                content = self._read_store_config_file()

            return content
        except (Exception,):
            logger.exception("Error while reading store configuration file")

        return "{}"

    @staticmethod
    def _read_store_config_file() -> str:
        with open(store_configs_path, 'r', encoding="utf-8") as file:
            return file.read()

    @staticmethod
    def _write_store_config_file(content: dict) -> None:
        try:
            with open(store_configs_path, "w") as file:
                file.write(json.dumps(content))

        except (Exception,):
            logger.exception("Failed to update store configuration file")

    def _get_pos_configs(
        self,
        store_config: Dict[str, str],
    ) -> Dict:
        try:
            pos_configs_json = self._download_from_url(
                url=self.configs.get(
                    "Store.POSConfigsJsonUrl",
                    "https://files.boh.e-deploy.com.br/POSConfigs.json"
                )
            ).content

            return self._update_fiscal_configs(
                pos_configs=json.loads(pos_configs_json.decode()),
                store_config=store_config,
            )
        except (Exception,):
            logger.exception("Error while downloading POSConfigs.json")
            return {}

    @staticmethod
    def _download_from_url(
        url: str,
    ) -> requests.Response:

        return requests.get(url, allow_redirects=True)

    def _update_fiscal_configs(
        self,
        pos_configs: dict,
        store_config: Dict[str, str],
    ) -> dict:

        fiscal_config = pos_configs.get("posConfigs", {}).get("fiscal")
        self._create_nfe_model_55_enabled(
            fiscal_config=fiscal_config,
            pos_configs=pos_configs,
            store_config=store_config,
        )
        return pos_configs

    def _create_nfe_model_55_enabled(
        self,
        fiscal_config: dict,
        pos_configs: dict,
        store_config: Dict[str, str],
    ) -> None:

        nfe_model_55_enabled = store_config.get("store", {}).get("nfe_model_55_enabled", False)
        pos_configs["posConfigs"]["fiscal"]["nfeModel55Enabled"] = nfe_model_55_enabled
        if nfe_model_55_enabled:
            return

        if fiscal_config is None:
            return

        activation_date = fiscal_config.get("nfeModel55Date", None)
        if activation_date is None:
            return

        pos_configs["posConfigs"]["fiscal"]["nfeModel55Enabled"] = self.verify_date(date_to_activate=activation_date)

    @staticmethod
    def verify_date(
        date_to_activate: Optional[str],
    ) -> bool:

        try:
            date_to_activate = datetime.strptime(date_to_activate, "%Y-%m-%d").date()
            today = date.today()
            return today >= date_to_activate

        except (Exception,):
            return False



from typing import (
    Dict,
    Any,
)

from store_config.repository import StoreConfigurationRepository


class StoreConfigurationService:

    def __init__(
        self,
        store_config_repository: StoreConfigurationRepository,
    ) -> None:

        self.store_config_repository = store_config_repository

    def get_configuration(
        self,
    ) -> Dict[str, Any]:

        return self.store_config_repository.get_current_configuration()

    def update_configuration(self) -> bool:

        return self.store_config_repository.update_store_configuration()



from ._GetConfigProcessor import GetConfigProcessor
from ._GetFullConfigProcessor import GetFullConfigProcessor
from ._SetConfigProcessor import SetConfigProcessor
from ._GetStoreConfigProcessor import GetStoreConfigProcessor
from ._UpdateStoreConfigProcessor import UpdateStoreConfigProcessor



from io import StringIO

from cfgtools import Configuration
from messagebus import (
    AckMessage,
    InternalErrorMessage,
)
from messageprocessorutil import UuidMessageProcessor


class GetFullConfigProcessor(UuidMessageProcessor):
    def __init__(self, store_config: Configuration):
        super().__init__()

        self.store_config = store_config

    def parse_data(self):
        return None

    def call_business(self, obj):
        return self.store_config

    def format_response(self, message_bus, message, event, input_obj, result: Configuration):
        with StringIO() as f:
            result.write(f)
            f.seek(0)
            config_str = f.read()

        message_bus.reply_message(
            message=message,
            reply_message=AckMessage(data=config_str.encode()),
        )

    def format_exception(self, message_bus, message, event, input_obj, exception):
        message_bus.reply_message(
            message=message,
            reply_message=InternalErrorMessage(ex=exception),
        )

    def format_parse_exception(self, message_bus, message, event, data, exception):
        self.format_exception(
            message_bus=message_bus,
            message=message,
            event=event,
            input_obj=data,
            exception=exception,
        )



from pathlib import Path

pos_dir = Path(__file__).parents[5]
storecfg_bundles_path = pos_dir / "data" / "server" / "bundles" / "storecfg"
store_configs_path = storecfg_bundles_path / "config.json"
jsons_dir = storecfg_bundles_path / "jsons"



from typing import (
    Any,
    Dict,
)

from store_config.service import StoreConfigurationService


class GetStoreConfigInteractor:

    def __init__(
        self,
        store_config_service: StoreConfigurationService,
    ) -> None:

        self.store_config_service = store_config_service

    def execute(
        self,
    ) -> Dict[str, Any]:

        return self.store_config_service.get_configuration()



def singleton(cls, *args, **kwargs):
    instances = {}

    def _singleton(*args, **kwargs):
        if cls not in instances:
            instances[cls] = cls(*args, **kwargs)

        return instances[cls]

    return _singleton
